@@ -1,12 +1,12 @@
 ;;; corfu-popupinfo.el --- Candidate information popup for Corfu -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2021-2025 Free Software Foundation, Inc.
 
 ;; Author: Yuwei Tian <fishtai0@gmail.com>, Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2022
-;; Version: 1.6
-;; Package-Requires: ((emacs "28.1") (compat "30") (corfu "1.6"))
+;; Version: 2.0
+;; Package-Requires: ((emacs "28.1") (compat "30") (corfu "2.0"))
 ;; URL: https://github.com/minad/corfu
 
 ;; This file is part of GNU Emacs.
@@ -182,8 +182,7 @@ all values are in pixels relative to the origin.  See
     (let ((old-buffers (buffer-list)) (buffer nil))
       (unwind-protect
           (when-let
-              ((extra (nth 4 completion-in-region--data))
-               (fun (plist-get extra :company-location))
+              ((fun (corfu--metadata-get 'company-location))
                ;; BUG: company-location may throw errors if location is not found
                (loc (ignore-errors (funcall fun candidate)))
                ((setq buffer
@@ -218,8 +217,7 @@ all values are in pixels relative to the origin.  See
 
 (defun corfu-popupinfo--get-documentation (candidate)
   "Get the documentation for CANDIDATE."
-  (when-let ((extra (nth 4 completion-in-region--data))
-             (fun (plist-get extra :company-doc-buffer))
+  (when-let ((fun (corfu--metadata-get 'company-doc-buffer))
              (res (save-excursion
                     (let ((inhibit-message t)
                           (message-log-max nil)
@@ -345,7 +343,7 @@ form (X Y WIDTH HEIGHT DIR)."
   (when corfu-popupinfo--timer
     (cancel-timer corfu-popupinfo--timer)
     (setq corfu-popupinfo--timer nil))
-  (when (and (corfu-popupinfo--visible-p corfu--frame))
+  (when (corfu-popupinfo--visible-p corfu--frame)
     (let* ((cand-changed
             (not (and (corfu-popupinfo--visible-p)
                       (corfu--equal-including-properties
@@ -495,7 +493,8 @@ not be displayed until this command is called again, even if
                             corfu-popupinfo-delay))
                    (corfu-popupinfo--toggle))
               (if (or (<= delay 0)
-                      (and (equal cand corfu-popupinfo--candidate)
+                      (and (corfu--equal-including-properties
+                            cand corfu-popupinfo--candidate)
                            (corfu-popupinfo--visible-p)))
                   (corfu-popupinfo--show cand)
                 (when (corfu-popupinfo--visible-p)
@@ -506,7 +505,8 @@ not be displayed until this command is called again, even if
                     (corfu-popupinfo--show corfu-popupinfo--candidate))))
                 (setq corfu-popupinfo--timer
                     (run-at-time delay nil #'corfu-popupinfo--show cand)))
-            (unless (equal cand corfu-popupinfo--candidate)
+            (unless (corfu--equal-including-properties
+                     cand corfu-popupinfo--candidate)
               (corfu-popupinfo--hide))))
       (corfu-popupinfo--hide))))
 
