@@ -1,11 +1,11 @@
 ;;; corfu.el --- COmpletion in Region FUnction -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2021-2025 Free Software Foundation, Inc.
 
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 1.6
+;; Version: 2.0
 ;; Package-Requires: ((emacs "28.1") (compat "30"))
 ;; URL: https://github.com/minad/corfu
 ;; Keywords: abbrev, convenience, matching, completion, text
@@ -338,6 +338,7 @@ settings `corfu-auto-delay', `corfu-auto-prefix' and
     (menu-bar-lines . 0)
     (tool-bar-lines . 0)
     (tab-bar-lines . 0)
+    (tab-bar-lines-keep-state . t)
     (no-other-frame . t)
     (unsplittable . t)
     (undecorated . t)
@@ -369,7 +370,7 @@ settings `corfu-auto-delay', `corfu-auto-prefix' and
   "Default child frame buffer parameters.")
 
 (defvar corfu--mouse-ignore-map
-  (let ((map (make-sparse-keymap)))
+  (let ((map (define-keymap "<touchscreen-begin>" #'ignore)))
     (dotimes (i 7)
       (dolist (k '(mouse down-mouse drag-mouse double-mouse triple-mouse))
         (keymap-set map (format "<%s-%s>" k (1+ i)) #'ignore)))
@@ -385,11 +386,10 @@ settings `corfu-auto-delay', `corfu-auto-prefix' and
 
 (defun corfu--capf-wrapper (fun &optional prefix)
   "Wrapper for `completion-at-point' FUN.
-The wrapper determines if the Capf is applicable at the current
-position and performs sanity checking on the returned result.
-For non-exclusive Capfs wrapper additionally checks if the
-current input can be completed successfully.  PREFIX is a prefix
-length override, set to t for manual completion."
+The wrapper determines if the Capf is applicable at the current position
+and performs sanity checking on the returned result.  For non-exclusive
+Capfs, the wrapper checks if the current input can be completed.  PREFIX
+is a prefix length override, which is t for manual completion."
   (pcase (funcall fun)
     ((and res `(,beg ,end ,table . ,plist))
      (and (integer-or-marker-p beg) ;; Valid Capf result
@@ -945,17 +945,15 @@ See `completion-in-region' for the arguments BEG, END, TABLE, PRED."
               (total (alist-get 'corfu--total state))
               (cands (alist-get 'corfu--candidates state)))
          (cond
-          ((= total 0)
-           ;; No candidates found for `newstr' -> Completion is finished.
-           (corfu--exit-function newstr 'finished nil))
-          ((= total 1)
+          ((<= total 1)
            ;; If completion is finished and cannot be extended further and
            ;; `corfu-on-exact-match' is not 'show, return 'finished.  Otherwise
            ;; setup the popup.
-           (if (or (eq corfu-on-exact-match 'show)
-                   (consp (completion-try-completion
-                           newstr table pred newpt
-                           (completion-metadata newstr table pred))))
+           (if (and (= total 1)
+                    (or (eq corfu-on-exact-match 'show)
+                        (consp (completion-try-completion
+                                newstr table pred newpt
+                                (completion-metadata newstr table pred)))))
                (corfu--setup beg end table pred)
              (corfu--exit-function newstr 'finished cands)))
           ;; Too many candidates for cycling -> Setup popup.
@@ -1067,7 +1065,8 @@ A scroll bar is displayed from LO to LO+BAR."
              (border (alist-get 'internal-border-width corfu--frame-parameters))
              (x (max 0 (min (+ (car edge) (- (or (car pos) 0) ml (* cw off) border))
                             (- (frame-pixel-width) width))))
-             (yb (+ (cadr edge) (window-tab-line-height) (or (cdr pos) 0) lh))
+             (yb (+ (cadr edge) (or (cdr pos) 0) lh
+                    (static-if (< emacs-major-version 31) (window-tab-line-height) 0)))
              (y (if (> (+ yb (* corfu-count ch) lh lh) (frame-pixel-height))
                     (- yb height lh border border)
                   yb))
@@ -1081,7 +1080,7 @@ A scroll bar is displayed from LO to LO+BAR."
         (unless (or (= right-fringe-width 0) (eq (get 'corfu--bar 'corfu--bmp) bmp))
           (put 'corfu--bar 'corfu--bmp bmp)
           (define-fringe-bitmap 'corfu--bar (vector (lognot bmp)) 1 mr '(top periodic))
-          (define-fringe-bitmap 'corfu--nil [])
+          (define-fringe-bitmap 'corfu--nil [0] 1 1)
           ;; Fringe bitmaps require symbol face specification, define internal face.
           (set-face-attribute (make-face 'corfu--cbar) nil
                               :inherit '(corfu--bar corfu-current)))
@@ -1107,9 +1106,7 @@ A scroll bar is displayed from LO to LO+BAR."
 
 (cl-defgeneric corfu--popup-support-p ()
   "Return non-nil if child frames are supported."
-  (or (display-graphic-p)
-      ;; Upcoming feature: Gerd Möllmann's child frame support on TTY.
-      (featurep 'tty-child-frames)))
+  (or (display-graphic-p) (featurep 'tty-child-frames)))
 
 (cl-defgeneric corfu--insert (status)
   "Insert current candidate, exit with STATUS if non-nil."
@@ -1128,9 +1125,8 @@ A scroll bar is displayed from LO to LO+BAR."
 
 (cl-defgeneric corfu--affixate (cands)
   "Annotate CANDS with annotation function."
-  (let* ((extras (nth 4 completion-in-region--data))
-         (dep (plist-get extras :company-deprecated))
-         (mf (let ((completion-extra-properties extras))
+  (let* ((dep (corfu--metadata-get 'company-deprecated))
+         (mf (let ((completion-extra-properties (nth 4 completion-in-region--data)))
                (run-hook-with-args-until-success 'corfu-margin-formatters corfu--metadata))))
     (setq cands
           (if-let ((aff (corfu--metadata-get 'affixation-function)))
@@ -1349,9 +1345,8 @@ input has been expanded."
       (corfu-complete)
     (pcase-let* ((`(,beg ,end ,table ,pred . ,_) completion-in-region--data)
                  (pt (max 0 (- (point) beg)))
-                 (str (buffer-substring-no-properties beg end))
-                 (metadata (completion-metadata (substring str 0 pt) table pred)))
-      (pcase (completion-try-completion str table pred pt metadata)
+                 (str (buffer-substring-no-properties beg end)))
+      (pcase (completion-try-completion str table pred pt corfu--metadata)
         ('t
          (goto-char end)
          (corfu--done str 'finished corfu--candidates)
